import QtQuick 2.0
import MuseScore 1.0

MuseScore {
      version:  "1.0"
      description: "This plugin halves the duration of all notes in the selection"
      menuPath: "Plugins.Halftime"

      // Copy all chords/notes/rests from the selection or entire score
      // this section leveraged from colornotes.qml
      function copyFromSelection(chordArray) {
            var cursor = curScore.newCursor();
            cursor.rewind(1); // beginning of selection
            var startStaff;
            var endStaff;
            var endTick;
            var fullScore = false;
            if (!cursor.segment) { // no selection
                  fullScore = true;
                  startStaff = 0; // start with 1st staff
                  endStaff = curScore.nstaves - 1; // and end with last
            } else {
                  startStaff = cursor.staffIdx;
                  cursor.rewind(2); // end of selection
                  if (cursor.tick == 0) { // selection includes last measure of score
                        endTick = curScore.lastSegment.tick + 1;
                  } else {
                        endTick = cursor.tick;
                  }
                  endStaff = cursor.staffIdx;
            }
            console.log(startStaff + " - " + endStaff + " - " + endTick)
            for (var staff = startStaff; staff <= endStaff; staff++) { // each staff
                  for (var voice = 0; voice < 4; voice++) { // each voice
                        cursor.rewind(1); // start over 
                        cursor.voice = voice; // select voice
                        cursor.staffIdx = staff; // select staff
                        if (fullScore) cursor.rewind(0); // unclear why this is needed

                        while (cursor.segment && (fullScore || cursor.tick < endTick)) { // each cursor position in selection
                              if (cursor.element){ // only copy chords/notes/rests...
                                    if (cursor.element.type == Element.CHORD) chordArray.push(cursor.element);
                                    if (cursor.element.type == Element.REST) chordArray.push(cursor.element);
                                    }
                              cursor.next();
                        }
                  }
            }
      }

      // Paste all chords/notes/rests with 2x duration to a new score
      function modifyAndPaste(chordArray) {
            // this section leveraged from createscore.qml
            var score = newScore("HalfTime", "piano", 32); // 32 measure limit
            score.addText("title", "HalfTime");
            var cursor = score.newCursor();
            var firstchord = 0;
            cursor.track = 0;
            cursor.rewind(0);
            var ts = newElement(Element.TIMESIG);
            ts.setSig(4,4);
            cursor.add(ts);

            for (var i = 0; i<chordArray.length; i++){ // each chord/note/rest
                  var chord = chordArray[i];
                  var dur = chord.durationType/15;

                  if (chord.track > cursor.track){ // track change detection
                        firstchord = i;
                        cursor.track = chord.track;
                        cursor.rewind(0);
                  }

                  if (chord.type == Element.CHORD){
                        //var newchord = clone(chord);  // wouldn't this be nice?!
                        var newchord = newElement(Element.CHORD);
                        for (var j = 0; j<chord.notes.length; j++){
                              var newnote = newElement(Element.NOTE);
                              newnote.pitch = chord.notes[j].pitch;
                              newnote.tpc1 = chord.notes[j].tpc1; // required for pitch display
                              newnote.tpc2 = chord.notes[j].tpc2;
                              newchord.add(newnote);
                        }
                  } else {
                        var newchord = newElement(Element.REST);
                  }
                  newchord.durationType = chord.durationType / 2;
                  {  // HACK 1 - add temporary note - this is the only way to populate measure without crashing
                        cursor.setDuration(dur / 2, 128); 
                        cursor.addNote(60); 
                  }
                  cursor.add(newchord); 
                  {  // HACK 2 - this is the only way to move forward by an exact amount
                        cursor.rewind(0);
                        for (var j = firstchord; j <= i; j++)  cursor.next();                  
                  }
            }
      }

      onRun: {
            if (typeof curScore === 'undefined')  Qt.quit();
            var chordArray = [];
            copyFromSelection(chordArray);
            modifyAndPaste(chordArray);
            Qt.quit();
         }
}
